//this file is part of eMule
//Copyright (C)2002 Merkur ( devs@emule-project.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#pragma once

#include "NetF/NetF_MilliSeconds.h"
#include "NetF/NetF_BytesPerSecond.h"

enum EReason
{
	REASON_NoNeededParts,
	REASON_Timeout,
	REASON_Disconnect,
	REASON_Cancel,
	REASON_Limit,
	REASON_Dropped,
	REASON_Other,
	REASON_None
};

enum ESmallFilePush
{
	SFP_None = 0,
	SFP_Nonlinear = 1,
	SFP_Full = 2
};

enum ETaskPriority
{
	TP_Low = 0,
	TP_BelowNormal = 1,
	TP_Normal = 2,
	TP_AboveNormal = 3,
	TP_High = 4
};

class CNetF
{
	// netfinity: Clean this later
	friend class CPPgNetF;
	friend class CPreferences;
	friend class CServerConnect;
	friend class CListenSocket;
	friend class CDownloadQueue;
	friend class CemuleDlg;
	friend class CUploadListCtrl;
	friend class CQueueListCtrl;
	friend class CClientCredits;
	friend class CUploadQueue;
	friend class CPartFile;
	friend class CUpDownClient;
	friend class CStatisticsDlg;

	// Status
	bool			m_bHighIdPossible;
	bool			m_bDownloadLimited;
	// Statistics
	uint32			m_iSessionSuccessfulDownloadNNP;
	uint32			m_iSessionSuccessfulDownloadLimit;
	uint32			m_iSessionSuccessfulDownloadTimeout;
	uint32			m_iSessionSuccessfulDownloadDisconnect;
	uint32			m_iSessionSuccessfulDownloadDropped;
	uint32			m_iSessionSuccessfulDownloadOther;
	uint32			m_iSessionFailedDownloadNNP;
	uint32			m_iSessionFailedDownloadLimit;
	uint32			m_iSessionFailedDownloadTimeout;
	uint32			m_iSessionFailedDownloadDisconnect;
	uint32			m_iSessionFailedDownloadDropped;
	uint32			m_iSessionFailedDownloadOther;
	uint32			m_iSessionSuccessfulUploadCancel;
	uint32			m_iSessionSuccessfulUploadTimeout;
	uint32			m_iSessionSuccessfulUploadDisconnect;
	uint32			m_iSessionSuccessfulUploadLimit;
	uint32			m_iSessionSuccessfulUploadOther;
	uint32			m_iSessionFailedUploadCancel;
	uint32			m_iSessionFailedUploadTimeout;
	uint32			m_iSessionFailedUploadDisconnect;
	uint32			m_iSessionFailedUploadLimit;
	uint32			m_iSessionFailedUploadOther;
	// Configuration
	ETaskPriority	m_iTaskPriority;
	ESmallFilePush	m_iSmallFilePush;
	bool			m_bMaxUpSlotsEnabled;
	bool			m_bMinUpDatarateEnabled;
	uint32			m_iMaxUpSlots;
	uint32			m_iMinUpDatarate;
	uint32			m_iUpSlotInterval;
	// Features
	bool			m_bPaybackQueue;
	bool			m_bFastQueueRotation;
	bool			m_bFreshmanCredits;
	bool			m_bDownloadHidden;
	bool			m_bRarePartCredits;
public:
	CNetF();
	void			Init(void);
	ESmallFilePush	GetSmallFilePush(void) const {return m_iSmallFilePush;}
	uint32			GetMaxUploadSlots(void) const {return (m_bMaxUpSlotsEnabled ? m_iMaxUpSlots : 0xFFFF);}
	CBytesPerSecond	GetMinUploadDatarate(void) const {return (m_bMinUpDatarateEnabled ? m_iMinUpDatarate : 1);}
	CMilliSeconds	GetMinUploadSlotInterval(void) const {return (m_iUpSlotInterval * 1000);}
	void			LoadPreferences(void);
	void			SavePreferences(void) const;
	void			UpdateTaskPriority(void);
};

extern CNetF theNetF;
