/** @file		NetF_MilliSeconds.h
 *  @brief	Type safe time calculations
 *  @author	netfinity
 */
#pragma once

#include "OtherFunctions.h"

/**
 *  @brief	Container for elapsed time in milliseconds
 */
class CMilliSeconds
{
public:
	CMilliSeconds(const sint32 dwTime = 0);
	CMilliSeconds(const CMilliSeconds& time);
	// Conversions
	sint32 GetValue() const;
	// Comparisations
	operator == (const CMilliSeconds& time) const;
	operator != (const CMilliSeconds& time) const;
	operator >= (const CMilliSeconds& time) const;
	operator <= (const CMilliSeconds& time) const;
	operator > (const CMilliSeconds& time) const;
	operator < (const CMilliSeconds& time) const;
	// Assignments
	CMilliSeconds operator += (const CMilliSeconds& time);
	CMilliSeconds operator -= (const CMilliSeconds& time);	
	// Arithmetics
	CMilliSeconds operator - (void) const;
	CMilliSeconds operator + (const CMilliSeconds& time) const;
	CMilliSeconds operator - (const CMilliSeconds& time) const;
	CMilliSeconds operator * (const float& multiplier) const;
	CMilliSeconds operator / (const float& divisor) const;
	CMilliSeconds operator % (const CMilliSeconds& time) const;
private:
	sint32	m_dwMilliSeconds;
};

//
//	inline operations
//

inline
CMilliSeconds::CMilliSeconds(const sint32 dwTime)
{
	m_dwMilliSeconds = dwTime;
}

inline
CMilliSeconds::CMilliSeconds(const CMilliSeconds& time)
{
	m_dwMilliSeconds = time.m_dwMilliSeconds;
}

inline
sint32
CMilliSeconds::GetValue() const
{
	return m_dwMilliSeconds;
}

inline
CMilliSeconds::operator == (const CMilliSeconds& time) const
{
	return m_dwMilliSeconds == time.m_dwMilliSeconds;
}

inline
CMilliSeconds::operator != (const CMilliSeconds& time) const
{
	return m_dwMilliSeconds != time.m_dwMilliSeconds;
}

inline
CMilliSeconds::operator >= (const CMilliSeconds& time) const
{
	return m_dwMilliSeconds >= time.m_dwMilliSeconds;
}

inline
CMilliSeconds::operator <= (const CMilliSeconds& time) const
{
	return m_dwMilliSeconds <= time.m_dwMilliSeconds;
}

inline
CMilliSeconds::operator > (const CMilliSeconds& time) const
{
	return m_dwMilliSeconds > time.m_dwMilliSeconds;
}

inline
CMilliSeconds::operator < (const CMilliSeconds& time) const
{
	return m_dwMilliSeconds < time.m_dwMilliSeconds;
}

inline
CMilliSeconds
CMilliSeconds::operator += (const CMilliSeconds& time)
{
	return m_dwMilliSeconds += time.m_dwMilliSeconds;
}

inline
CMilliSeconds
CMilliSeconds::operator -= (const CMilliSeconds& time)
{
	return m_dwMilliSeconds += time.m_dwMilliSeconds;
}

inline
CMilliSeconds
CMilliSeconds::operator - (void) const
{
	return CMilliSeconds(-m_dwMilliSeconds);
}

inline
CMilliSeconds
CMilliSeconds::operator + (const CMilliSeconds& time) const
{
	return CMilliSeconds(m_dwMilliSeconds + time.m_dwMilliSeconds);
}

inline
CMilliSeconds
CMilliSeconds::operator - (const CMilliSeconds& time) const
{
	return CMilliSeconds(m_dwMilliSeconds - time.m_dwMilliSeconds);
}

inline
CMilliSeconds
CMilliSeconds::operator * (const float& multiplier) const
{
	return CMilliSeconds(m_dwMilliSeconds * multiplier);
}

inline
CMilliSeconds
CMilliSeconds::operator / (const float& divisor) const
{
	return CMilliSeconds(m_dwMilliSeconds / divisor);
}

inline
CMilliSeconds
CMilliSeconds::operator % (const CMilliSeconds& time) const
{
	return CMilliSeconds(m_dwMilliSeconds % time.m_dwMilliSeconds);
}

//
//	Special arithmetics
//

inline
CMilliSeconds
operator * (const float& multiplier, const CMilliSeconds& time)
{
	return CMilliSeconds(multiplier * time.GetValue());
}

inline
float
operator / (const CMilliSeconds& time1, const CMilliSeconds& time2)
{
	return ((float) time1.GetValue() / (float) time2.GetValue());
}

//
//	
//

inline
void
Sleep(const CMilliSeconds& time)
{
	::Sleep(time.GetValue());
}

inline
CString
CastSecondsToHM(const CMilliSeconds& time)
{
	if (time >= 0)
		return ::CastSecondsToHM(time.GetValue() / 1000);
	else
		return _T("-") + ::CastSecondsToHM(-time.GetValue() / 1000);
}
