/** @file		NetF_Tick.h
 *  @brief	Keeps track of elapsed time
 *  @author	netfinity
 */
#pragma once

#include "NetF_MilliSeconds.h"
#include "NetF_Bytes.h"
#include "NetF_BytesPerSecond.h"

class CSeconds;
class CTick;

/**
 *  @brief	Container for elapsed time in seconds
 */
class CSeconds
{
public:
	CSeconds(const sint32 dwTime = 0)
		{m_dwSeconds = dwTime;}
	CSeconds(const CSeconds& time)
		{m_dwSeconds = time.m_dwSeconds;}
	// Time conversion
	CSeconds(const CMilliSeconds& time)
		{m_dwSeconds = time.GetValue() / 1000;}
	CSeconds(const time_t& time)
		{m_dwSeconds = time;}
	operator CMilliSeconds() const
		{return CMilliSeconds(m_dwSeconds * 1000);}
	operator time_t() const
		{return time_t(m_dwSeconds);}
	// Assignments
	CSeconds operator += (const CSeconds& time)
		{return CSeconds(m_dwSeconds += time.m_dwSeconds);}
	// Aritmetics
	CSeconds operator + (const CSeconds& time) const
		{return CSeconds(m_dwSeconds + time.m_dwSeconds);}
	CSeconds operator - (const CSeconds& time) const
		{return CSeconds(m_dwSeconds - time.m_dwSeconds);}
private:
	sint32			m_dwSeconds;
};

/**
 *  @brief	Replaces the use of DWORD or UINT for counting elapsed time
 */
class CTick
{
public:
	CTick()
	{
		m_dwTickCount = 0;
	}
	CTick(const CTick& tick)
	{
		m_dwTickCount = tick.m_dwTickCount;
	}
	CMilliSeconds	Elapsed() const
	{
		if (m_dwTickCount == 0)
			return 0;
		else
			return static_cast<CMilliSeconds>((sint32) ::GetTickCount() - (sint32) m_dwTickCount);
	}
	CMilliSeconds	Elapsed(const CTick& tick) const
	{
		if (m_dwTickCount == 0)
			throw CString(_T(__FUNCTION__) _T("; tick not set!"));
		else
			return static_cast<CMilliSeconds>((sint32) tick.m_dwTickCount - (sint32) m_dwTickCount);
	}
	bool			IsTickSet() const
	{
		return (m_dwTickCount != 0);
	}
	void			SetCurrentTick()
	{
		m_dwTickCount = ::GetTickCount();
		if (m_dwTickCount == 0)
			m_dwTickCount = 1;
	}
	static CTick	GetCurrentTick()
	{
		CTick tick;
		tick.m_dwTickCount = ::GetTickCount();
		if (tick.m_dwTickCount == 0)
			tick.m_dwTickCount = 1;
		return tick;
	}
	void			UnsetTick()
	{
		m_dwTickCount = 0;
	}
	// Aritmetics
	CTick operator + (const CMilliSeconds& time) const
	{
		CTick tick;
		if (m_dwTickCount != 0)
			tick.m_dwTickCount = (uint32) ((sint32) m_dwTickCount + (sint32) time.GetValue());
		else
			throw CString(_T(__FUNCTION__) _T("; tick not set!"));
		return tick;
	}
	CMilliSeconds operator - (const CTick& tick) const
	{
		if (m_dwTickCount == 0 || tick.m_dwTickCount == 0)
			throw CString(_T(__FUNCTION__) _T("; tick not set!"));
		else
			return ((sint32) m_dwTickCount - (sint32) tick.m_dwTickCount);
	}
	CTick operator - (const CMilliSeconds& time) const
	{
		CTick tick;
		if (m_dwTickCount != 0)
			tick.m_dwTickCount = (uint32) ((sint32) m_dwTickCount - (sint32) time.GetValue());
		else
			throw CString(_T(__FUNCTION__) _T("; tick not set!"));
		return tick;
	}
private:
	uint32			m_dwTickCount;
};

//
// 
//

